import java.io.IOException;
import java.io.InputStream;
import org.apache.log4j.Level;
import org.jetel.component.AbstractGenericTransform;
import org.jetel.data.DataRecord;
import org.jetel.exception.ComponentNotReadyException;
import org.jetel.exception.ConfigurationStatus;
import org.jetel.exception.ConfigurationStatus.Priority;
import org.jetel.exception.ConfigurationStatus.Severity;
import org.jetel.exception.JetelRuntimeException;
import org.jetel.metadata.DataRecordMetadata;

import com.datastax.driver.core.Cluster;
import com.datastax.driver.core.ColumnDefinitions;
import com.datastax.driver.core.ResultSet;
import com.datastax.driver.core.Row;
import com.datastax.driver.core.Session;

/**
 * This is an example custom reader. It shows how you can
 *  create records using a data source.
 */
public class myCassandraReader extends AbstractGenericTransform {

	@Override
	public void execute() {
		// get component properties
		String node = getProperties().getStringProperty("cassandraNode");
		String keySapce = getProperties().getStringProperty("cassandraKeySpace");
		String sql = getProperties().getStringProperty("cassandraSQL");
		
		// open connection to Cassandra
		Cluster cluster = Cluster.builder().addContactPoint(node).build();
		Session session = cluster.connect(keySapce);
		
		// run SQL query
		ResultSet results = session.execute(sql);
		ColumnDefinitions columns = results.getColumnDefinitions();
		
		// initialize output record
		DataRecord outputRecord = outRecords[0];
		
		// fetch data from Cassandra
		for (Row row: results) {
			int columnOffset = 0;
			String columnType;
			String columnName;
			
			for (ColumnDefinitions.Definition column: columns) {
				columnType = column.getType().asFunctionParameterString();
				columnName = column.getName();
				
				switch (columnType) {
					case "int":
						outputRecord.getField(columnOffset).setValue(row.getInt(columnOffset));
						break;
					case "varchar":
						outputRecord.getField(columnOffset).setValue(row.getString(columnOffset));
						break;
					default:
						throw new JetelRuntimeException("ERROR: Unknown data type \"" + columnType + "\" of column \"" + columnName + "\".");
				}
				
				columnOffset++;
			}
			
			writeRecordToPort(0, outputRecord);
			outputRecord.reset();
		}
		
		// close connection to Cassandra
		cluster.close();
		
		
		
		/** This code is an example. Replace it with your custom code. */

		/** This is how you can read component properties. */
		//String customValue = getProperties().getStringProperty("myCustomPropertyName");

		/** You can log messages. */
		//getLogger().log(Level.DEBUG, "Custom property resolved to: " + customValue);

		/** The source you want to read from. This could be a local file, a database table... */
		//String[] source = new String[] {"first", "second", "third"};

		/** Output record prepared for writing to output port 0. */
		//DataRecord record = outRecords[0];

		/** Write record to output port 0 for each part of our source. */
		/*
		for (String part : source) {
			record.getField("myMetadataFieldName").setValue(part);
			writeRecordToPort(0, record);
			record.reset();
		}
		*/

		/** Advanced example: Reading bytes from a file and sending each byte in an output record on port 0. */
		/*
		try (InputStream in = getInputStream("data-in/file.txt")) {
			int byteFromFile;
			while ((byteFromFile = in.read()) != -1) {
				record.getField("singleByte").setValue(byteFromFile);
				writeRecordToPort(0, record);
				record.reset();
			}
		} catch (IOException e) {
			throw new JetelRuntimeException("Problem with reading input file", e);
		}
		*/
	}

	@Override
	public ConfigurationStatus checkConfig(ConfigurationStatus status) {
		super.checkConfig(status);

		/** This way you can check connected edges and their metadata. */
		/*
		if (getComponent().getOutPorts().size() < 1) {
			status.add("Output port must be connected!", Severity.ERROR, getComponent(), Priority.NORMAL);
			return status;
		}

		DataRecordMetadata outMetadata = getComponent().getOutputPort(0).getMetadata();
		if (outMetadata == null) {
			status.add("Metadata on output port not specified!", Severity.ERROR, getComponent(), Priority.NORMAL);
			return status;
		}

		if (outMetadata.getFieldPosition("myMetadataFieldName") == -1) {
			status.add("Incompatible output metadata!", Severity.ERROR, getComponent(), Priority.NORMAL);
		}
		*/
		return status;
	}

	@Override
	public void init() {
		super.init();
	}

	@Override
	public void preExecute() throws ComponentNotReadyException {
		super.preExecute();
	}

	@Override
	public void postExecute() throws ComponentNotReadyException {
		super.postExecute();
	}
}
